//
//  ControleurFenetrePrincipale.m
// This file is part of COLLATINVS Mac.
//
//  COLLATINVS is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  COLLATINVS is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with COLLATINVS; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  Created by Alain Baud on 17/10/08.
//

#import "ControleurFenetrePrincipale.h"

//#include "libcollatinus.h"
//#include "frequences.h">

@implementation ControleurFenetrePrincipale

@synthesize texteALemmatiser,vocabulaire,onglet,champRecherche,alphabetice,cumTextusVocibus;




-(void)awakeFromNib
{
	// Lecture des préférences utilisateur pour la prise en compte de la langue et de la morphologie
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
		int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];
	
	
	NSNotificationCenter *nc;
	nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self
		   selector:@selector(changementLangue:) 
			   name:@"GBOChangementLangue"
			 object:nil];
	
	[nc addObserver:self 
		   selector:@selector(changementNiveauLangue:) 
			   name:@"GBOChangementNiveauLangue"
			 object:nil];
	
	[nc addObserver:self 
		   selector:@selector(changementMorphologie:) 
			   name:@"GBOChangementAvecMorphologie"
			 object:nil];
	
	[nc addObserver:self
		   selector:@selector(modifTexte:) 
			   name:@"NSTextDidChangeNotification"
			 object:nil];	
	

	// Définition du nom de fichier par défaut
	NSString *path = [[NSBundle mainBundle] pathForResource:@"lucretia" ofType:@"txt"];
	// Ouverture du fichier par défaut
	NSString *s=[NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
	
	
	modeleCollatinus=[[GBOModeleCollatinus alloc]initAvecTexteSource:s avecMorphologie:avecMorphologie];

	
	
	
	// création d'un attribut "tooltip" attaché à chaque mot
	switch (indexLangue)
	{
		case 0: // en français
			[texteALemmatiser insertText:[modeleCollatinus frTexteSource]];
			break;
		case 1: // en allemand
			[texteALemmatiser insertText:[modeleCollatinus deTexteSource]];
			break;
		case 2: // en anglais
			[texteALemmatiser insertText:[modeleCollatinus ukTexteSource]];
			break;
	
	}
	
	// Remonte le scroll au début du texte
	NSRange r={0,0};
	[texteALemmatiser scrollRangeToVisible:r];		
}

-(void) dealloc
{

NSNotificationCenter *nc;
nc=[NSNotificationCenter defaultCenter];
	[nc removeObserver:self];
	[modeleCollatinus release];
	[super dealloc];
}
	
	
-(IBAction) appuiBoutonLemmatiser:(id)sender
{
	
	// Lecture des préférences utilisateur pour la prise en compte de la rareté des mots
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
	int minRaritas=[[preferencesUtilisateur valueForKey:@"fréquence"]integerValue];
	int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];

	[modeleCollatinus toutLemmatiserAvecFrequence:minRaritas];
    [vocabulaire setString:[modeleCollatinus vocabulaireMisEnFormeEn:[[onglet selectedTabViewItem] label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];
	[preferencesUtilisateur release];
}


-(IBAction) ouvrirFichier:(id) sender
{
	NSOpenPanel * panelOuvrirFichier = [NSOpenPanel openPanel];
	NSError **erreur;
	[panelOuvrirFichier setAllowedFileTypes:[NSArray arrayWithObjects:@"txt",nil]];
	
	if([panelOuvrirFichier runModal]==NSFileHandlingPanelOKButton)	
	{
		NSString *texteOuvert=[NSString stringWithContentsOfFile:[panelOuvrirFichier filename] encoding:NSUTF8StringEncoding error:erreur];
		[texteALemmatiser setString:texteOuvert];
		// Force l'envoi d'une notification de changement de texte car par défaut aucune notificqtion n'est envoyée...
		[[NSNotificationCenter defaultCenter] postNotificationName:@"NSTextDidChangeNotification" object:self];
	}
	else
	{
	// Appui sur le bouton annuler de la boîte de dialogue
	}
	[panelOuvrirFichier release];
}

-(IBAction) enregistrerFichierSous:(id) sender
{
	// Lecture des préférences utilisateur pour la prise en compte de la langue
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
	int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];
	
	NSSavePanel * panelEnregistrerFichierSous = [NSSavePanel savePanel];
	NSMutableString *texteAEnregistrer=[[NSMutableString alloc]init];
	
	if([[[onglet selectedTabViewItem] label] isEqualToString:@"HTML"])
	{
		// En-tête fichier HTML
		[panelEnregistrerFichierSous setRequiredFileType:@"html"];
		[texteAEnregistrer appendString:@"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"];
		[texteAEnregistrer appendString:@"<head>\n"];
		[texteAEnregistrer appendString:@"<meta name=\"generator\" content=\"Collatinus Mac\">\n"];
		[texteAEnregistrer appendString:@"</head>\n\n"];
		[texteAEnregistrer appendString:@"<body>\n"];
		
	}
	else if([[[onglet selectedTabViewItem] label] isEqualToString:@"LaTeX"])
	{
		[panelEnregistrerFichierSous setRequiredFileType:@"tex"];
		// En-tête fichier Latex
		[texteAEnregistrer appendString:@"\\documentclass[12pt]{article}\n"];
		[texteAEnregistrer appendString:@"\\usepackage[T1]{fontenc}\n"];
		[texteAEnregistrer appendString:@"\\usepackage[latin1]{inputenc}\n"];
		[texteAEnregistrer appendString:@"\\usepackage{geometry}\n"];
		[texteAEnregistrer appendString:@"\\usepackage[frenchb]{babel}\n"];
		[texteAEnregistrer appendString:@"\\geometry{a4paper, left=1in, right=1in, top=1in, bottom=1in}\n"];
		[texteAEnregistrer appendString:@"\\begin{document}\n"];
		
	}
	else 
	{
		[panelEnregistrerFichierSous setRequiredFileType:@"txt"];
	}

	[panelEnregistrerFichierSous setTitle:@"Scribere ut..."];
	[panelEnregistrerFichierSous setPrompt:@"Scribere"];
	[panelEnregistrerFichierSous setNameFieldLabel:@"Scribere ut..."];
	if ([panelEnregistrerFichierSous runModal]==NSFileHandlingPanelOKButton)
		{
		
		// Préparation du texte à enregistrer à partir de la liste des mots à traduire
		if (![[[onglet selectedTabViewItem] label] isEqualToString:@"Flexio"])
			{
			[texteAEnregistrer appendString:[texteALemmatiser string]];
			[texteAEnregistrer appendString:@"\n\n"];
			if([[[onglet selectedTabViewItem] label] isEqualToString:@"HTML"])
			{
				[texteAEnregistrer appendString:@"<ul>\n"];
			}
			if([[[onglet selectedTabViewItem] label] isEqualToString:@"LaTeX"])	
			{
				[texteAEnregistrer appendString:@"\\begin{itemize}\n"];
			}	
				// Liste des mots traduits
				[texteAEnregistrer appendString:[modeleCollatinus vocabulaireMisEnFormeEn:[[onglet selectedTabViewItem] label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];
			
				if([[[onglet selectedTabViewItem] label] isEqualToString:@"HTML"])
			{
				[texteAEnregistrer appendString:@"</ul>\n"];
			[texteAEnregistrer appendString:@"</body>"];
			}
				
				if([[[onglet selectedTabViewItem] label] isEqualToString:@"LaTeX"])	
				{
					[texteAEnregistrer appendString:@"\\end{itemize}\n"];
				[texteAEnregistrer appendString:@"\\end{document}"];
				}	
				
			}
		else
			{
			// Pour l'onglet Flexio, on enregistrera directement ce qui est affiché à l'écran
			[texteAEnregistrer appendString:[vocabulaire string]];
			}
		
		NSError **erreur;
		// Enregistrement des données dans le fichier choisi
		if (![texteAEnregistrer writeToFile:[panelEnregistrerFichierSous filename] atomically:NO encoding:NSUTF8StringEncoding error:erreur])
			{
			}
		[texteAEnregistrer release];
		}
		else
		{
		// Code en cas d'appui sur le bouton annuler de la boîte de dialogue
		}
	[panelEnregistrerFichierSous release];
	[preferencesUtilisateur release];
}




-(IBAction) ordreAlphabetique:(id)sender;
{
	// Lecture des préférences utilisateur pour la prise en compte de la langue
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
	int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];
	
	[vocabulaire setString:[modeleCollatinus vocabulaireMisEnFormeEn:[[onglet selectedTabViewItem] label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];
	
	[preferencesUtilisateur release];
}

-(IBAction) avecMotATraduire:(id)sender;
{
	// Lecture des préférences utilisateur pour la prise en compte de la langue
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
	int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];
	
	[vocabulaire setString:[modeleCollatinus vocabulaireMisEnFormeEn:[[onglet selectedTabViewItem] label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];
	
	[preferencesUtilisateur release];
}


-(IBAction) augmenterTailleCaracteres:(id)sender
{
	// Récupère la police actuelle de la zone de texte
	NSFont *policeActuelle = [texteALemmatiser font];
	// Récupère les propriétés de la police
	NSFontDescriptor *descriptionPoliceActuelle=[policeActuelle fontDescriptor];
	// Récupère la taille de la police
	NSNumber *taille=[descriptionPoliceActuelle objectForKey:NSFontSizeAttribute];
	
	// Augmente la taille de la police de 1 dans les deux zones de texte
	[texteALemmatiser setFont:[[NSFontManager sharedFontManager] 
						   convertFont:policeActuelle toSize:[taille floatValue]+1]];
	[vocabulaire setFont:[[NSFontManager sharedFontManager] 
							   convertFont:policeActuelle toSize:[taille floatValue]+1]];
	
	[policeActuelle release];
	[descriptionPoliceActuelle release];
	[taille release];

}

-(IBAction) diminuerTailleCaracteres:(id)sender
{
	// Récupère la police actuelle de la zone de texte
	NSFont *policeActuelle = [texteALemmatiser font];
	// Récupère les propriétés de la police
	NSFontDescriptor *descriptionPoliceActuelle=[policeActuelle fontDescriptor];
	// Récupère la taille de la police
	NSNumber *taille=[descriptionPoliceActuelle objectForKey:NSFontSizeAttribute];
	
	// Augmente la taille de la police de 1 dans les deux zones de texte
	[texteALemmatiser setFont:[[NSFontManager sharedFontManager] 
							   convertFont:policeActuelle toSize:[taille floatValue]-1]];
	[vocabulaire setFont:[[NSFontManager sharedFontManager] 
						  convertFont:policeActuelle toSize:[taille floatValue]-1]];
	
	[policeActuelle release];
	[descriptionPoliceActuelle release];
	[taille release];
}

- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
// Lecture des préférences utilisateur pour la prise en compte de la langue
id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];

	if([[[onglet selectedTabViewItem] label] isEqualTo:@"Flexio"])
	{
		[vocabulaire setString:@""];
	}		
		else
		{
	[vocabulaire setString:[modeleCollatinus vocabulaireMisEnFormeEn:[tabViewItem label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];	

		}
	[preferencesUtilisateur release];
}

- (void)textViewDidChangeSelection:(NSNotification *)aNotification
{
	
	// Lecture des préférences utilisateur pour la prise en compte de la langue
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
	int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];
	
	NSRange plage=[texteALemmatiser selectedRange];
	
	if([[[onglet selectedTabViewItem] label] isEqualTo:@"Flexio"])
	{
		[vocabulaire setString:[modeleCollatinus afficheFlexioDeLaPlage:plage]];
	}
	else
	{
	[modeleCollatinus ajoutMotDansTraduction:plage];	
				
		[vocabulaire setString:[modeleCollatinus vocabulaireMisEnFormeEn:[[onglet selectedTabViewItem] label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];
	}
		[preferencesUtilisateur release];
}



-(void)changementLangue:(NSNotification *) note
	{
		// Lecture des préférences utilisateur pour la prise en compte de la langue
		id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
		int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
		BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];
		
		[texteALemmatiser setString:@""];
		
		// création d'un attribut "tooltip" attaché à chaque mot
		switch (indexLangue)
		{
			case 0: // en français
				[texteALemmatiser insertText:[modeleCollatinus frTexteSource]];
				break;
			case 1: // en allemand
				[texteALemmatiser insertText:[modeleCollatinus deTexteSource]];
				break;
			case 2: // en anglais
				[texteALemmatiser insertText:[modeleCollatinus ukTexteSource]];
				break;
				
		}
		
		[vocabulaire setString:[modeleCollatinus vocabulaireMisEnFormeEn:[[onglet selectedTabViewItem] label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];
		
		[preferencesUtilisateur release];
		}


-(void)changementNiveauLangue:(NSNotification *) note
{
	NSLog(@"Notification de changement de niveau de langue reçue");
}

-(void)changementMorphologie:(NSNotification *) note
{
	NSLog(@"Changement morphologie");
	// Lecture des préférences utilisateur pour la prise en compte de la langue
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
	int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];
	
	[modeleCollatinus changeTexteSource:[texteALemmatiser string] avecMorphologie:avecMorphologie];
	
	[texteALemmatiser setString:@""];
	
	// création d'un attribut "tooltip" attaché à chaque mot
	switch (indexLangue)
	{
		case 0: // en français
			[texteALemmatiser insertText:[modeleCollatinus frTexteSource]];
			break;
		case 1: // en allemand
			[texteALemmatiser insertText:[modeleCollatinus deTexteSource]];
			break;
		case 2: // en anglais
			[texteALemmatiser insertText:[modeleCollatinus ukTexteSource]];
			break;
			
	}
	
	[vocabulaire setString:[modeleCollatinus vocabulaireMisEnFormeEn:[[onglet selectedTabViewItem] label] avecIndexLangue:indexLangue avecMotDuTexte:[cumTextusVocibus state] dansLOrdreAlphabetique:[alphabetice state] avecMorphologie:avecMorphologie]];
	
	[preferencesUtilisateur release];
}


-(void)modifTexte:(NSNotification *) note
{
	// Lecture des préférences utilisateur pour la prise en compte de la langue
	id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
	BOOL avecMorphologie=[[preferencesUtilisateur valueForKey:@"morphologie"]boolValue];

	if ([[texteALemmatiser string] isEqualToString:[[modeleCollatinus frTexteSource] string]])	{
	}
	else
	{
		NSArray *plagesActuelle=[texteALemmatiser selectedRanges];
		[modeleCollatinus changeTexteSource:[texteALemmatiser string] avecMorphologie:avecMorphologie];
		
		// Lecture des préférences utilisateur pour la prise en compte de la langue
		id preferencesUtilisateur = [[NSUserDefaultsController sharedUserDefaultsController] values];
		int indexLangue=[[preferencesUtilisateur valueForKey:@"langue"]integerValue];
		
		[texteALemmatiser setString:@""];
		
		// création d'un attribut "tooltip" attaché à chaque mot
		switch (indexLangue)
		{
			case 0: // en français
				[texteALemmatiser insertText:[modeleCollatinus frTexteSource]];
				break;
			case 1: // en allemand
				[texteALemmatiser insertText:[modeleCollatinus deTexteSource]];
				break;
			case 2: // en anglais
				[texteALemmatiser insertText:[modeleCollatinus ukTexteSource]];
				break;
				
		}
		[texteALemmatiser setSelectedRanges:plagesActuelle];
		[plagesActuelle release];
	}
}

- (IBAction)chercherDansTexte:(id)sender
{
	NSRange r=[[[texteALemmatiser textStorage] string]rangeOfString:[champRecherche stringValue] options:NSCaseInsensitiveSearch|NSDiacriticInsensitiveSearch];
	
	if (r.location!=NSNotFound) 
		{
		[texteALemmatiser scrollRangeToVisible:r];
		[texteALemmatiser showFindIndicatorForRange:r];
		}
	}

@end





