//
//  GBOModeleCollatinus.m
//  This file is part of COLLATINVS Mac.
//
//  COLLATINVS is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  COLLATINVS is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with COLLATINVS; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  Created by Alain Baud on 27/03/09.
//

#import "GBOModeleCollatinus.h"
#import "StringLatin.h"

@implementation GBOModeleCollatinus

@synthesize frTexteSource,deTexteSource,ukTexteSource,formes,motsATraduire;

-(id)init
{
	[super init];
	if (self !=nil)
	{
		// Chargement des lexiques
		
		NSString *frFichierLexicum;
		NSString *deFichierLexicum;
		NSString *ukFichierLexicum;
		
		NSBundle *thisBundle = [NSBundle mainBundle];
		
		frFichierLexicum = [thisBundle pathForResource:@"lemmata" ofType:@"fr"];
		deFichierLexicum = [thisBundle pathForResource:@"lemmata" ofType:@"de"];
		ukFichierLexicum = [thisBundle pathForResource:@"lemmata" ofType:@"uk"];
		
		frLexicum = new TLexicum ([frFichierLexicum UTF8String]);
		deLexicum = new TLexicum ([deFichierLexicum UTF8String]);
		ukLexicum = new TLexicum ([ukFichierLexicum UTF8String]);
		
		self.frTexteSource=@"";
		self.deTexteSource=@"";
		self.ukTexteSource=@"";
		
		motsATraduire=[[NSMutableArray alloc]init];
		
		[frFichierLexicum release];
		[deFichierLexicum release];
		[ukFichierLexicum release];
		
		[thisBundle release];
	}
	
	return self;
}


-(id)initAvecTexteSource:(NSString *)texte avecMorphologie:(BOOL)avecMorphologie
{
	[super init];
	if (self !=nil)
		{
			// Chargement des lexiques
			
			NSString *frFichierLexicum;
			NSString *deFichierLexicum;
			NSString *ukFichierLexicum;
			
			NSBundle *thisBundle = [NSBundle mainBundle];
			
			frFichierLexicum = [thisBundle pathForResource:@"lemmata" ofType:@"fr"];
			deFichierLexicum = [thisBundle pathForResource:@"lemmata" ofType:@"de"];
			ukFichierLexicum = [thisBundle pathForResource:@"lemmata" ofType:@"uk"];
			
			[thisBundle release];
			
			frLexicum = new TLexicum ([frFichierLexicum UTF8String]);
			deLexicum = new TLexicum ([deFichierLexicum UTF8String]);
			ukLexicum = new TLexicum ([ukFichierLexicum UTF8String]);
			
			[frFichierLexicum release];
			[deFichierLexicum release];
			[ukFichierLexicum release];
			
			
			
			motsATraduire=[[NSMutableArray alloc]init];
			
			// Création de la liste des mots (formes)
			
			NSCharacterSet *jeuDeCaracteresAConserver=[NSCharacterSet letterCharacterSet];
			NSCharacterSet *jeuDeCaracteresASupprimer=[jeuDeCaracteresAConserver invertedSet];
			
			// Séparation des mots
			self.formes=[texte componentsSeparatedByCharactersInSet:jeuDeCaracteresASupprimer];
			
			[jeuDeCaracteresAConserver release];
			[jeuDeCaracteresASupprimer release];
			
			// Création des Tooltips
			// Les Tooltips sont contenu directement dans les NSMutableAttributerString sous forme de NSDictionary  
			
			
			// Création du texte pour toutes les langues
			NSMutableAttributedString *frAs, *deAs, *ukAs;
			frAs=[[NSMutableAttributedString alloc] initWithString:texte];
			deAs=[[NSMutableAttributedString alloc] initWithString:texte];
			ukAs=[[NSMutableAttributedString alloc] initWithString:texte];
			
			NSDictionary *frAttr, *deAttr, *ukAttr;
			
			// Enumération de tous les mots du texte
			NSEnumerator *i = [formes objectEnumerator];	
			NSMutableString *mot=[[NSMutableArray alloc]init]; 
			
			NSUInteger debutPlage=0;
			NSRange plage={debutPlage,[texte length]-1};
			
			while((mot=[i nextObject]))
			{	
				
				if(mot.length>0) // n'ajoute rien si la longueur de mot est null 
				{	
					frAttr=[[NSDictionary alloc] initWithObjectsAndKeys:[self traduire:mot avecIndexLangue:0 avecMorphologie:avecMorphologie], NSToolTipAttributeName, nil];
					deAttr=[[NSDictionary alloc] initWithObjectsAndKeys:[self traduire:mot avecIndexLangue:1 avecMorphologie:avecMorphologie], NSToolTipAttributeName, nil];
					ukAttr=[[NSDictionary alloc] initWithObjectsAndKeys:[self traduire:mot avecIndexLangue:2 avecMorphologie:avecMorphologie], NSToolTipAttributeName, nil];
					
					[frAs setAttributes:frAttr range:[texte rangeOfString:mot options:nil range:plage]];
					[deAs setAttributes:deAttr range:[texte rangeOfString:mot options:nil range:plage]];
					[ukAs setAttributes:ukAttr range:[texte rangeOfString:mot options:nil range:plage]];
					
					debutPlage=[texte rangeOfString:mot options:nil range:plage].location;
					plage=NSMakeRange(debutPlage,[texte length]-1-debutPlage);
					
					[frAttr release];
					[deAttr release];
					[ukAttr release];
					
				}
			}
			
			
			frTexteSource=frAs;
			deTexteSource=deAs;
			ukTexteSource=ukAs;
			
			[frAs release];
			[deAs release];
			[ukAs release];
			
			[i release];
			[mot release];
			
		}
	return self;
}




-(void)dealloc
{

	delete frLexicum;
	delete deLexicum;
	delete ukLexicum;
	[motsATraduire release];
	[super dealloc];
}

-(NSString *) traduire:(NSMutableString *)motATraduire avecIndexLangue:(int)indexLangue avecMorphologie:(BOOL)avecMorphologie
{
	switch (indexLangue) {

			
			
		case 0: { // Traduction en Français	
			if (avecMorphologie)
			{
			std::string frTraduction=frLexicum->lemmatiseM([motATraduire UTF8String]);
			return [[NSString stringWithUTF8String:frTraduction.c_str()]substringFromIndex:0];
			}
			else
			{
				std::string frTraduction=frLexicum->lemmatise([motATraduire UTF8String]);
			return [[NSString stringWithUTF8String:frTraduction.c_str()]substringFromIndex:0];
			}
			
			break;
		}
		case 1:	{//Traduction en Allemand
			if (avecMorphologie)
			{
			std::string deTraduction=deLexicum->lemmatiseM([motATraduire UTF8String]);
			return [NSString stringWithUTF8String:deTraduction.c_str()];
			}
			else
			{
			std::string deTraduction=deLexicum->lemmatise([motATraduire UTF8String]);
			return [NSString stringWithUTF8String:deTraduction.c_str()];
			}
			
			break;
		}
		case 2:  {//Traduction en Anglais
			
			if (avecMorphologie)
			{
			std::string ukTraduction=ukLexicum->lemmatiseM([motATraduire UTF8String]);
			return [NSString stringWithUTF8String:ukTraduction.c_str()];
			}
			else
			{
			std::string ukTraduction=ukLexicum->lemmatise([motATraduire UTF8String]);
			return [NSString stringWithUTF8String:ukTraduction.c_str()];
			}
				
			break;
		}
	}	
	return @"err";
}


-(void)toutLemmatiserAvecFrequence:(int)frequence 
			{
				NSEnumerator *i = [formes objectEnumerator];
				NSMutableString *uox=[[NSMutableString alloc]init];
				
				while ((uox = [i nextObject])) 
				{
					if(uox.length>0) // n'ajoute rien si la longueur de uox est null 
					{	
						
						if ([[[self traduire:uox avecIndexLangue:0 avecMorphologie:NO] prima]frequentia]< frequence)
						{
							continue;
						}
										
						[motsATraduire addObject:uox];
					}
				}
				[i release];
				[uox release];
}


-(void)changeTexteSource:(NSString *)nouveauTexteSource avecMorphologie:(BOOL)avecMorphologie
{
	
	[formes release];
	if (![nouveauTexteSource isEqualToString:[[self frTexteSource]string]])
	{
	[motsATraduire removeAllObjects];
	}
	if ([nouveauTexteSource length]>0)
	{
	
	// Création de la liste des mots (formes)
	
	NSCharacterSet *jeuDeCaracteresAConserver=[NSCharacterSet letterCharacterSet];
	NSCharacterSet *jeuDeCaracteresASupprimer=[jeuDeCaracteresAConserver invertedSet];
	
	// Séparation des mots
	self.formes=[nouveauTexteSource componentsSeparatedByCharactersInSet:jeuDeCaracteresASupprimer];
	
		[jeuDeCaracteresAConserver release];
		[jeuDeCaracteresASupprimer release];
	
	// Création des Tooltips
	// Les Tooltips sont contenu directement dans les NSMutableAttributerString sous forme de NSDictionary  
	
	
	// Création du texte pour toutes les langues
	NSMutableAttributedString *frAs, *deAs, *ukAs;
	frAs=[[NSMutableAttributedString alloc] initWithString:nouveauTexteSource];
	deAs=[[NSMutableAttributedString alloc] initWithString:nouveauTexteSource];
	ukAs=[[NSMutableAttributedString alloc] initWithString:nouveauTexteSource];
	
	NSDictionary *frAttr, *deAttr, *ukAttr;
	
	// Enumération de tous les mots du texte
	NSEnumerator *i = [formes objectEnumerator];	
	NSMutableString *mot=[[NSMutableArray alloc]init]; 
	
	NSUInteger debutPlage=0;
	NSRange plage={debutPlage,[nouveauTexteSource length]-1};
	
	while((mot=[i nextObject]))
	{	
		
		if(mot.length>0) // n'ajoute rien si la longueur de mot est null 
		{	
			frAttr=[[NSDictionary alloc] initWithObjectsAndKeys:[self traduire:mot avecIndexLangue:0 avecMorphologie:avecMorphologie], NSToolTipAttributeName, nil];
			deAttr=[[NSDictionary alloc] initWithObjectsAndKeys:[self traduire:mot avecIndexLangue:1 avecMorphologie:avecMorphologie], NSToolTipAttributeName, nil];
			ukAttr=[[NSDictionary alloc] initWithObjectsAndKeys:[self traduire:mot avecIndexLangue:2 avecMorphologie:avecMorphologie], NSToolTipAttributeName, nil];
			
			[frAs setAttributes:frAttr range:[nouveauTexteSource rangeOfString:mot options:nil range:plage]];
			[deAs setAttributes:deAttr range:[nouveauTexteSource rangeOfString:mot options:nil range:plage]];
			[ukAs setAttributes:ukAttr range:[nouveauTexteSource rangeOfString:mot options:nil range:plage]];
			
			debutPlage=[nouveauTexteSource rangeOfString:mot options:nil range:plage].location;
			plage=NSMakeRange(debutPlage,[nouveauTexteSource length]-1-debutPlage);
			[frAttr release];
			[deAttr release];
			[ukAttr release];
			
		}
	}
	
	
	frTexteSource=frAs;
	deTexteSource=deAs;
	ukTexteSource=ukAs;
		
		[i release];
		[mot release];

	}
}

-(NSString *)vocabulaireMisEnFormeEn:(NSString *)typeMiseEnForme avecIndexLangue:(int)indexLangue avecMotDuTexte:(BOOL)avecMotDuTexte dansLOrdreAlphabetique:(BOOL)dansLOrdreAlphabetique avecMorphologie:(BOOL)avecMorphologie
{
NSMutableString *texte=[[NSMutableString alloc]init];
	NSArray *listeMotsATraduire;
	NSMutableString *motPrecedent=@"";
	if (dansLOrdreAlphabetique)
	listeMotsATraduire=[motsATraduire sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];	
		else
	listeMotsATraduire=motsATraduire;
	if ([typeMiseEnForme isEqualToString:@"Textus"])
	{
		[texte appendString:@""];
		for (NSMutableString* mot in listeMotsATraduire)
		{
			if ([mot compare:motPrecedent]==NSOrderedSame)
			{
			motPrecedent=mot;
			}
			else 
			{
				if (avecMotDuTexte)
					{
						[texte appendString:mot];
						[texte appendString:@"\n"];

					}
			[texte appendString:[self traduire:mot avecIndexLangue:indexLangue avecMorphologie:avecMorphologie]];
			[texte appendString:@"\n"];
			[motPrecedent release];
			motPrecedent=mot;
			}
		}
		return texte;
	}
			 else if ([typeMiseEnForme isEqualToString:@"HTML"])
	{
		
				for (NSMutableString* mot in listeMotsATraduire)
				{
					if ([mot compare:motPrecedent]==NSOrderedSame)
					{
						motPrecedent=mot;
					}
					else
					{
					[texte appendString:@"<li>"];
					if (avecMotDuTexte)
						{
						[texte appendString:@"<b>"];
						[texte appendString:mot];
						[texte appendString:@"</b>\n"];

						}
					
						
					[texte appendString:[self traduire:mot avecIndexLangue:indexLangue avecMorphologie:avecMorphologie]];
					[texte appendString:@"</li>\n"];
					[motPrecedent release];
					motPrecedent=mot;
					}
				}
					return texte;
			}
					 else if ([typeMiseEnForme isEqualToString:@"LaTeX"])
	{
			
			for (NSMutableString* mot in listeMotsATraduire)
						{
							if ([mot compare:motPrecedent]==NSOrderedSame)
							{
								motPrecedent=mot;
							}
							else
							{
							[texte appendString:@"\\item "];				
							if (avecMotDuTexte)
							{
								[texte appendString:@"\\textbf{"];
								[texte appendString:mot];
								[texte appendString:@"}"];
							[texte appendString:@"\n"];
							}
							[texte appendString:[self traduire:mot avecIndexLangue:indexLangue avecMorphologie:avecMorphologie]];
							[texte appendString:@"\n"];
							[motPrecedent release];
							motPrecedent=mot;
							}
							 }
							 return texte;
					}
else
	return NULL;
}





-(void)ajoutMotDansTraduction:(NSRange)plage
{
	if (plage.length>0)
	{
		NSAttributedString *as=[[NSAttributedString alloc]init];
		NSString *s=[[NSString alloc]init];
		as=[frTexteSource attributedSubstringFromRange:plage];
		s=[as string];
		if ( [formes indexOfObject:s]==NSNotFound)
		{
		// Pas un mot complet
		}
		else
		{
			[motsATraduire addObject:s];
		}
		[as release];
		[s release];
	}
	
}

-(NSString *)afficheFlexioDeLaPlage:(NSRange)plage
{
	if (plage.length>0)
	{
		NSAttributedString *as=[[NSAttributedString alloc]init];
		NSString *s=[[NSString alloc]init];
		as=[frTexteSource attributedSubstringFromRange:plage];
		s=[as string];
		NSLog(s);
		if ( [formes indexOfObject:s]==NSNotFound)
		{
		// Pas un mot complet
		}
		else
		{
			std::string ss=frLexicum->lemmatise([s UTF8String]);
			NSString *traduction=[NSString stringWithUTF8String:ss.c_str()];
			NSArray *decompositionMot=[traduction componentsSeparatedByString:@","];
			NSLog([[decompositionMot objectAtIndex:0] substringFromIndex:1]);
			Tentree *e=frLexicum->entree([[[decompositionMot objectAtIndex:0] substringFromIndex:1] UTF8String]);
			if (!e->est_vide())
				{
					std::string flexio=frLexicum->flechis(e);
					return [NSString stringWithUTF8String:flexio.c_str()];
				}
			else
			{
				NSString* messageErreur=[NSString stringWithFormat:@"id %@ non intellexi.",s];
				return (messageErreur);
			}
				//[motsATraduire addObject:s];
		}
		[as release];
		[s release];
	}		
	return (@"");
	
	
	
}

@end
